(* TEST_BELOW
(* Blank lines added here to preserve locations. *)




*)

 #1 "pr6604.ml"

(* TEST
 flags = "-dparsetree";
 ocamlc_byte_exit_status = "2";
 setup-ocamlc.byte-build-env;
 ocamlc.byte;
 check-ocamlc.byte-output;
*)
