(* TEST
 ocamldoc with latex;
*)

(** Test for level 0 headings

  {1 Level 1}

   Standard heading levels start at 1.

  {0 Level 0}
  A level 0 heading is guaranteed to be at the same level that
  the main heading of the module.

  This setup allows users to start their standard heading at level 1 rather
  than 2, without losing the ability to add global level heading,
  when, if ever, such heading is warranted

 *)
