(* TEST
 toplevel;
*)

(* Test the printing of strings in the terminal *)
"\n\t\r\b";;

{|"\'|};;

" !#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[]^_`\
abcdefghijklmnopqrstuvwxyz{|}~";;

"\x00\x01\x02\x03\x04\x05\x06\x07\x0B\x0C\x0E\x0F\
 \x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F\
 \x7F";;

"\"Ἀχιλλεύς\r\n天照\tब्रह्मन्\t𒄑 𒂆 𒈦 𒄑 𒂆  𒈦\\";;

"ایدهآل";;
