(* TEST_BELOW *)




(* A test for stack backtraces *)

exception Error of string

let rec f msg n =
  if n = 0 then raise(Error msg) else 1 + f msg (n-1)

let g msg =
  try
    f msg 5
  with Error "a" -> print_string "a"; print_newline(); 0
     | Error "b" as exn -> print_string "b"; print_newline(); raise exn
     | Error "c" -> raise (Error "c")

let _ =
  ignore (g Sys.argv.(1))

(* TEST
 flags = "-g";
 ocamlrunparam += ",b=1";
*)
