(**************************************************************************)
(*                                                                        *)
(*                                 OCaml                                  *)
(*                                                                        *)
(*                 David Allsopp, OCaml Labs, Cambridge.                  *)
(*                                                                        *)
(*   Copyright 2020 David Allsopp Ltd.                                    *)
(*                                                                        *)
(*   All rights reserved.  This file is distributed under the terms of    *)
(*   the GNU Lesser General Public License version 2.1, with the          *)
(*   special exception on linking described in the file LICENSE.          *)
(*                                                                        *)
(**************************************************************************)

(* Dummy implementations for when the Unix library isn't built *)
let has_symlink () = false
let symlink ?to_dir:_ _ _ = invalid_arg "symlink not available"
let chmod _ _ = invalid_arg "chmod not available"
let gettimeofday () = invalid_arg "gettimeofday not available"
