(* TEST
 toplevel;
*)

(3; 2;;

begin 3; 2;;

List.(3; 2;;

simple_expr.(3; 2;;

simple_expr.[3; 2;;

simple_expr.%[3;;

simple_expr.%(3;;

simple_expr.%{3;;

foo.Bar.%[3;;

foo.Bar.%(3;;

foo.Bar.%{3;;

simple_expr.{3, 2;;

{ x = 3; y;;

List.{ x = 3; y ;;

[| 3; 2;;

List.[|3; 2;;

[3; 2;;

List.[3; 2;;

{< x = 3; y; ;;

List.{< x = 3; y ;;

(module struct end :;;

List.(module struct end :;;

(=;;

while true do ();;

for _ = e1 to e2 do ();;
