(**************************************************************************)
(*                                                                        *)
(*                                 OCaml                                  *)
(*                                                                        *)
(*                 David Allsopp, OCaml Labs, Cambridge.                  *)
(*                                                                        *)
(*   Copyright 2020 David Allsopp Ltd.                                    *)
(*                                                                        *)
(*   All rights reserved.  This file is distributed under the terms of    *)
(*   the GNU Lesser General Public License version 2.1, with the          *)
(*   special exception on linking described in the file LICENSE.          *)
(*                                                                        *)
(**************************************************************************)

(** Functions imported from Unix. They are explicitly here to remove the
    temptation to use the Unix module directly in ocamltest. *)

val has_symlink : unit -> bool
val symlink : ?to_dir:bool -> string -> string -> unit
val chmod : string -> int -> unit
val gettimeofday : unit -> float
