(* TEST
 include tool-ocaml-lib;
 flags = "-w -a";
 ocaml_script_as_argument = "true";
 setup-ocaml-build-env;
 ocaml;
*)

(* empty file *)

(**
       0 ATOM0
       1 SETGLOBAL T000
       3 STOP
**)
