(* TEST_BELOW
(* Blank lines added here to preserve locations. *)
*)

(* Nothing to see here, parse.ml dictates that these be printed as regular
   "Syntax error". *)

try foo () with ;;

(3 : );;

(3 :> );;

(* TEST
 toplevel;
*)
