(* TEST
 include tool-ocaml-lib;
 flags = "-w -a";
 ocaml_script_as_argument = "true";
 setup-ocaml-build-env;
 ocaml;
*)

3;;

(**
       0 CONST3
       1 ATOM0
       2 SETGLOBAL T010-const3
       4 STOP
**)
