(**************************************************************************)
(*                                                                        *)
(*                                 OCaml                                  *)
(*                                                                        *)
(*             Sebastien Hinderer, projet Gallium, INRIA Paris            *)
(*                                                                        *)
(*   Copyright 2017 Institut National de Recherche en Informatique et     *)
(*     en Automatique.                                                    *)
(*                                                                        *)
(*   All rights reserved.  This file is distributed under the terms of    *)
(*   the GNU Lesser General Public License version 2.1, with the          *)
(*   special exception on linking described in the file LICENSE.          *)
(*                                                                        *)
(**************************************************************************)

(* Interface to the Tsl_lexer module *)

val token : Lexing.lexbuf -> Tsl_parser.token
val is_test : Lexing.lexbuf -> bool
val modifier :
    Lexing.lexbuf -> string * [`Remove | `Add of string | `Append of string]

val has_comments : bool ref
