(* TEST
 ocamldoc with latex;
*)

(** Ten comments for tests *)

(** {6 A first comments for title } *)

(** {7 A subsection for ocamldoc *} *)

(** {7 Bis } *)

(** {7 Ter } *)

(** {6 A new section } *)

(** {7 And its subsection } *)

(** {7 Encore } *)

(** Encore! Encore! *)


(**/**)
module Silence : sig
  (** At last *)
end

(**/**)

(** {7 With strange aeons } *)

module End : sig end
