(**************************************************************************)
(*                                                                        *)
(*                                 OCaml                                  *)
(*                                                                        *)
(*             Sebastien Hinderer, projet Gallium, INRIA Paris            *)
(*                                                                        *)
(*   Copyright 2016 Institut National de Recherche en Informatique et     *)
(*     en Automatique.                                                    *)
(*                                                                        *)
(*   All rights reserved.  This file is distributed under the terms of    *)
(*   the GNU Lesser General Public License version 2.1, with the          *)
(*   special exception on linking described in the file LICENSE.          *)
(*                                                                        *)
(**************************************************************************)

(* Description of ocamltest's command-line options *)

val log_to_stderr : bool

val files_to_test : string list

val promote : bool

val default_timeout : int

val usage : string

val find_test_dirs : string list

val list_tests : string list

val keep_test_dir_on_success : bool

val show_timings : bool

val translate : bool
val style : Translate.style
val compact : bool
